@@ -1,51 +1,26 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
+
+
 package com.openexchange.fileitem.impl;
 
 import static com.openexchange.fileitem.impl.FileItemUtils.LOG;
@@ -348,21 +323,26 @@ public class FileItemDatabase implements ChangeExecListener {
         int index = 0;
 
         final String insertContentSql = STR_BUILDER("INSERT INTO ").append(TBL_CONTENT(groupId, null)).
-            append(" (FileStoreNumber,FileStoreId,SubGroupId,FileId) VALUES (?,?,?,?)").toString();
-
+            append(" (FileStoreNumber,FileStoreId,SubGroupId,FileId) VALUES (?,?,?,?)").
+            append(" ON DUPLICATE KEY UPDATE").
+            append(" FileStoreNumber=?, FileStoreId=?, SubGroupId=?, FileId=?").toString();
         final StringBuilder insertPropertiesBuilderStart = STR_BUILDER("INSERT INTO ").append(TBL_PROPERTIES(groupId, null)).
             append(" (FileStoreNumber,FileStoreId,CreateDate,ModificationDate,Length");
-
         final StringBuilder insertPropertiesBuilderEnd = STR_BUILDER(" VALUES (?,?,?,?,?");
+        final StringBuilder updatePropertiesBuilder = STR_BUILDER(" ON DUPLICATE KEY UPDATE ").
+            append(" FileStoreNumber=?,FileStoreId=?,CreateDate=?,ModificationDate=?,Length=?");
 
         if (customKeys.size() > 0) {
             for (final Object curCustomKey : customKeys) {
                 insertPropertiesBuilderStart.append(',').append(curCustomKey);
                 insertPropertiesBuilderEnd.append(",?");
+
+                updatePropertiesBuilder.append(',').append(curCustomKey).append("=?");
             }
         }
 
-        final String insertPropertiesSql = insertPropertiesBuilderStart.append(") ").append(insertPropertiesBuilderEnd).append(')').toString();
+        final String insertOrUpdatePropertiesSql = insertPropertiesBuilderStart.append(") ").append(insertPropertiesBuilderEnd).append(')').
+            append(updatePropertiesBuilder).toString();
 
         for (int sleepTimeoutMillis = 1; sleepTimeoutMillis <= 1; sleepTimeoutMillis <<= 1) {
             Connection usedCon = null;
@@ -371,24 +351,48 @@ public class FileItemDatabase implements ChangeExecListener {
             // fileElement and FileStoreData are valid
             try (final ConnectionWrapper conWrapper = getWriteConnectionWrapper();
                  final PreparedStatement fcStmt = conWrapper.prepareStatement(insertContentSql);
-                 final PreparedStatement fpStmt = conWrapper.prepareStatement(insertPropertiesSql)) {
+                 final PreparedStatement fpStmt = conWrapper.prepareStatement(insertOrUpdatePropertiesSql)) {
+
+                final int fileStoreNumber = fileStoreData.getFileStoreNumber();
+                final String fileStoreId = fileStoreData.getFileStoreId();
+                final long createDateMillis = fileItemProperties.getCreateDateMillis();
+                final long modificationDateMillis = fileItemProperties.getModificationDateMillis();
+                final long length = fileItemProperties.getLength();
 
                 usedCon = conWrapper.getConnection();
 
                 // create entry in Content table
-                fcStmt.setInt(index = 1, fileStoreData.getFileStoreNumber());
-                fcStmt.setString(++index, fileStoreData.getFileStoreId());
+                fcStmt.setInt(index = 1, fileStoreNumber);
+                fcStmt.setString(++index, fileStoreId);
+                fcStmt.setString(++index, subGroupId);
+                fcStmt.setString(++index, fileId);
+
+                fcStmt.setInt(++index, fileStoreNumber);
+                fcStmt.setString(++index, fileStoreId);
                 fcStmt.setString(++index, subGroupId);
                 fcStmt.setString(++index, fileId);
 
                 // create properties
-                fpStmt.setInt(index = 1, fileStoreData.getFileStoreNumber());
-                fpStmt.setString(++index, fileStoreData.getFileStoreId());
-                fpStmt.setLong(++index, fileItemProperties.getCreateDateMillis());
-                fpStmt.setLong(++index, fileItemProperties.getModificationDateMillis());
-                fpStmt.setLong(++index, fileItemProperties.getLength());
+                fpStmt.setInt(index = 1, fileStoreNumber);
+                fpStmt.setString(++index, fileStoreId);
+                fpStmt.setLong(++index, createDateMillis);
+                fpStmt.setLong(++index, modificationDateMillis);
+                fpStmt.setLong(++index, length);
 
-                // set given key values
+                // set given insert key custom values
+                if (customKeys.size() > 0) {
+                    for (final Object curCustomKey : customKeys) {
+                        fpStmt.setString(++index, fileItemProperties.getCustomKeyValue(curCustomKey.toString()));
+                    }
+                }
+
+                fpStmt.setInt(++index, fileStoreNumber);
+                fpStmt.setString(++index, fileStoreId);
+                fpStmt.setLong(++index, createDateMillis);
+                fpStmt.setLong(++index, modificationDateMillis);
+                fpStmt.setLong(++index, length);
+
+                // set given update custom values
                 if (customKeys.size() > 0) {
                     for (final Object curCustomKey : customKeys) {
                         fpStmt.setString(++index, fileItemProperties.getCustomKeyValue(curCustomKey.toString()));
@@ -855,6 +859,21 @@ public class FileItemDatabase implements ChangeExecListener {
         return implExecuteDelete(groupId, whereStr, deletedFileStoreDataList);
     }
 
+    /**
+     * @param groupId
+     * @param fileStoreData
+     * @param deletedFileStoreDataList
+     * @return
+     * @throws FileItemException
+     */
+    public int deleteByFileStoreData(@NonNull final String groupId, @NonNull final FileStoreData fileStoreData, @NonNull final List<FileStoreData> deletedFileStoreDataList) throws FileItemException {
+        final String whereStr = STR_BUILDER("fc.FileStoreNumber=").append("'").append(fileStoreData.getFileStoreNumber()).append("'").
+                append(" AND").
+                append(" fc.FileStoreId=").append("'").append(fileStoreData.getFileStoreId()).append("'").toString();
+
+        return implExecuteDelete(groupId, whereStr, deletedFileStoreDataList);
+    }
+
     /**
      * @param groupId
      * @param subGroupIds
@@ -1260,10 +1279,10 @@ public class FileItemDatabase implements ChangeExecListener {
      */
     private void implCreateConnectionPools() throws FileItemException {
         if (null != m_configService) {
-            final String readDriverClass = m_configService.getProperty(CFG_KEY("readDriverClass"));
-            final String writeDriverClass = m_configService.getProperty(CFG_KEY("writeDriverClass"));
-            final String jdbcReadUrl = m_configService.getProperty(CFG_KEY("readUrl"));
-            final String jdbcWriteUrl = m_configService.getProperty(CFG_KEY("writeUrl"));
+            final String readDriverClass = m_configService.getProperty(CFG_KEY_1("readDriverClass"));
+            final String writeDriverClass = m_configService.getProperty(CFG_KEY_1("writeDriverClass"));
+            final String jdbcReadUrl = m_configService.getProperty(CFG_KEY_1("readUrl"));
+            final String jdbcWriteUrl = m_configService.getProperty(CFG_KEY_1("writeUrl"));
 
             try {
                 Class.forName(readDriverClass);
@@ -1277,30 +1296,33 @@ public class FileItemDatabase implements ChangeExecListener {
             }
 
             try {
-                final int poolMaxSize = m_configService.getIntProperty(CFG_KEY("connectionpool.maxPoolSize"), HikariSetup.DEFAULT_POOL_MAXSIZE);
-                final int poolConnectTimeoutMillis = m_configService.getIntProperty(CFG_KEY("connectionpool.connectTimeout"), (int) HikariSetup.DEFAULT_POOL_CONNECTTIMEOUTMILLIS);
-                final int poolIdleTimeoutMillis = m_configService.getIntProperty(CFG_KEY("connectionpool.idleTimeout"), (int) HikariSetup.DEFAULT_POOL_IDLETIMEOUTMILLIS);
+                final int poolMaxSize = m_configService.getIntProperty(CFG_KEY_1("connectionpool.maxPoolSize"), HikariSetup.DEFAULT_POOL_MAXSIZE);
+                final int poolConnectTimeoutMillis = m_configService.getIntProperty(CFG_KEY_1("connectionpool.connectTimeout"), (int) HikariSetup.DEFAULT_POOL_CONNECTTIMEOUTMILLIS);
+                final int poolIdleTimeoutMillis = m_configService.getIntProperty(CFG_KEY_1("connectionpool.idleTimeout"), (int) HikariSetup.DEFAULT_POOL_IDLETIMEOUTMILLIS);
+                final int poolMaxLifetimeMillis = m_configService.getIntProperty(CFG_KEY_1("connectionpool.maxLifetime"), (int) HikariSetup.DEFAULT_POOL_MAX_LIFETIMEMILLIS);
                 Properties dbProperties = null;
 
                 // DB read properties and DataSource
                 implFillDBConnectionPropertiesFromConfigItems(
                     dbProperties = new Properties(),
-                    m_configService.getProperties(new WildcardNamePropertyFilter(CFG_KEY("readProperty.*"))));
+                    m_configService.getProperties(new WildcardNamePropertyFilter(CFG_KEY_1("readProperty.*"))),
+                    m_configService.getProperties(new WildcardNamePropertyFilter(CFG_KEY_2("readProperty.*"))));
 
                 m_readConnectionPool = new HikariSetup(readDriverClass, jdbcReadUrl,
                     dbProperties.getProperty("user"), dbProperties.getProperty("password"),
                     dbProperties,
-                    poolMaxSize, poolConnectTimeoutMillis, poolIdleTimeoutMillis).getConnectionPool(true);
+                    poolMaxSize, poolConnectTimeoutMillis, poolIdleTimeoutMillis, poolMaxLifetimeMillis).getConnectionPool(true);
 
                 // DB write properties and DataSource
                 implFillDBConnectionPropertiesFromConfigItems(
                     dbProperties = new Properties(),
-                    m_configService.getProperties(new WildcardNamePropertyFilter(CFG_KEY("writeProperty.*"))));
+                    m_configService.getProperties(new WildcardNamePropertyFilter(CFG_KEY_1("writeProperty.*"))),
+                    m_configService.getProperties(new WildcardNamePropertyFilter(CFG_KEY_2("writeProperty.*"))));
 
                 m_writeConnectionPool = new HikariSetup(writeDriverClass, jdbcWriteUrl,
                     dbProperties.getProperty("user"), dbProperties.getProperty("password"),
                     dbProperties,
-                    poolMaxSize, poolConnectTimeoutMillis, poolIdleTimeoutMillis).getConnectionPool(false);
+                    poolMaxSize, poolConnectTimeoutMillis, poolIdleTimeoutMillis, poolMaxLifetimeMillis).getConnectionPool(false);
             } catch (OXException e) {
                 throw new FileItemException(e);
             }
@@ -1647,15 +1669,17 @@ public class FileItemDatabase implements ChangeExecListener {
      * @param dbProperties
      * @param configItems
      */
-    private static void implFillDBConnectionPropertiesFromConfigItems(@NonNull final Properties dbProperties, @NonNull Map<String, String> configItems) {
-        for (final String curKey : configItems.keySet()) {
-            final String dbConfigPair = configItems.get(curKey);
+    private static void implFillDBConnectionPropertiesFromConfigItems(@NonNull final Properties dbProperties, @NonNull Map<String, String>... configItems) {
+        for (final Map<String, String> curConfigItems : configItems) {
+            for (final String curKey : curConfigItems.keySet()) {
+                final String dbConfigPair = curConfigItems.get(curKey);
 
-            if (isNotEmpty(dbConfigPair)) {
-                final int assignPos = dbConfigPair.indexOf('=');
+                if (isNotEmpty(dbConfigPair)) {
+                    final int assignPos = dbConfigPair.indexOf('=');
 
-                if ((assignPos > 0) && (assignPos < (dbConfigPair.length() - 1))) {
-                    dbProperties.put(dbConfigPair.substring(0, assignPos), dbConfigPair.substring(assignPos + 1));
+                    if ((assignPos > 0) && (assignPos < (dbConfigPair.length() - 1))) {
+                        dbProperties.put(dbConfigPair.substring(0, assignPos), dbConfigPair.substring(assignPos + 1));
+                    }
                 }
             }
         }
@@ -1920,13 +1944,23 @@ public class FileItemDatabase implements ChangeExecListener {
     }
 
     /**
-     * CFG_VALUE
+     * CFG_VALUE_1 (with config prefix FILEITEM_CONFIG_PREFIX_1)
      *
      * @param key
      * @return
      */
-    final private static @NonNull String CFG_KEY(@NonNull final String unprefixedKey) {
-        return STR_BUILDER(FILEITEM_CONFIG_PREFIX).append(unprefixedKey).toString();
+    final private static @NonNull String CFG_KEY_1(final String unprefixedKey) {
+        return STR_BUILDER(FILEITEM_CONFIG_PREFIX_1).append(unprefixedKey).toString();
+    }
+
+    /**
+     * CFG_VALUE_2 (with config prefix FILEITEM_CONFIG_PREFIX_2)
+     *
+     * @param key
+     * @return
+     */
+    final private static @NonNull String CFG_KEY_2(final String unprefixedKey) {
+        return STR_BUILDER(FILEITEM_CONFIG_PREFIX_2).append(unprefixedKey).toString();
     }
 
     /**
@@ -1949,7 +1983,8 @@ public class FileItemDatabase implements ChangeExecListener {
     final private static int STR_BUILDER_DEFAULT_CAPACITY = 128;
     final private static int STR_BUILDER_LARGE_CAPACITY = 256;
 
-    final private static String FILEITEM_CONFIG_PREFIX = "com.openexchange.fileItem.";
+    final private static String FILEITEM_CONFIG_PREFIX_1 = "com.openexchange.fileItem.";
+    final private static String FILEITEM_CONFIG_PREFIX_2 = "com.openexchange.fileitem.";
 
     // - Members ---------------------------------------------------------------
 
