@@ -1,72 +1,42 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
+
+
 package com.openexchange.fileitem.impl;
 
 import static org.apache.commons.lang.StringUtils.isEmpty;
 import static org.apache.commons.lang.StringUtils.isNotEmpty;
-
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InvalidClassException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
+import java.util.Optional;
 import java.util.Properties;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.function.Consumer;
-
 import org.apache.commons.collections.BidiMap;
-import org.apache.commons.io.IOUtils;
 import org.json.JSONObject;
-
 import com.openexchange.annotation.NonNull;
 import com.openexchange.annotation.Nullable;
 import com.openexchange.exception.OXException;
@@ -103,15 +73,15 @@ public class FileItemService implements IFileItemService {
     /**
      * Initializes a new {@link FileItemService}.
      */
-    public FileItemService(@NonNull final BidiMap fileStoreMap, @NonNull final FileItemDatabase database) throws FileItemException {
+    public FileItemService(final Optional<BidiMap> fileStoreMapOpt, final Optional<FileItemDatabase> databaseOpt) {
 
         super();
 
         // check for valid FileStores and database
-        m_fileStorageMap = fileStoreMap;
-        m_database = database;
+        m_fileStorageMap = fileStoreMapOpt.isPresent() ? fileStoreMapOpt.get() : null;
+        m_database = databaseOpt.isPresent() ? databaseOpt.get() : null;
 
-        if (m_fileStorageMap.size() > 0) {
+        if ((null != m_fileStorageMap) && (m_fileStorageMap.size() > 0) && (null != m_database)) {
             final int fileStoreCount =m_fileStorageMap.size();
             int curFileStoreArrayPos = -1;
 
@@ -120,15 +90,24 @@ public class FileItemService implements IFileItemService {
             for (final Object curFileStorage : m_fileStorageMap.keySet()) {
                 m_fileStorageArray[++curFileStoreArrayPos] = (FileStorage) curFileStorage;
             }
-        } else {
-            throw new FileItemException(new InvalidClassException("File store could not be initialized"));
-        }
 
-        (m_fileStoreRemover = new FileStoreRemover(this)).start();
+            (m_fileStoreRemover = new FileStoreRemover(this)).start();
+            m_valid = true;
+        } else {
+            m_valid = false;
+        }
     }
 
     // - IFileItemService ------------------------------------------------------
 
+    /* (non-Javadoc)
+     * @see com.openexchange.imageconverter.api.IFileItemService#isValid()
+     */
+    @Override
+    public boolean isValid() {
+        return m_valid;
+    }
+
     /* (non-Javadoc)
      * @see com.openexchange.imageconverter.api.IFileItemService#getDBType()
      */
@@ -525,7 +504,7 @@ public class FileItemService implements IFileItemService {
                 try {
                     ret.open();
                 } catch (Exception e) {
-                    IOUtils.closeQuietly(ret);
+                    FileItemUtils.close(ret);
                     ret = null;
 
                     throw new FileItemException(e);
@@ -565,7 +544,7 @@ public class FileItemService implements IFileItemService {
                 try {
                     ret.open();
                 } catch (Exception e) {
-                    IOUtils.closeQuietly(ret);
+                    FileItemUtils.close(ret);
                     ret = null;
 
                     throw new FileItemException(e);
@@ -636,7 +615,7 @@ public class FileItemService implements IFileItemService {
      * @param fileItem
      * @return
      */
-    protected synchronized FileStoreData createNewFileStoreFile(@NonNull IFileItem fileItem,
+    protected FileStoreData createNewFileStoreFile(@NonNull IFileItem fileItem,
         @NonNull FileItemProperties fileItemProperties, InputStream fileContent) throws IOException {
 
         final int fileStoreCount = m_fileStorageArray.length;
@@ -646,8 +625,7 @@ public class FileItemService implements IFileItemService {
             // walk over all available FileStores round robin wise
             // and create a new FileStore file
             for (int tryNumber = 0; (null == ret) && (tryNumber < fileStoreCount); ++tryNumber) {
-                final int nextFileStorePos = (m_curFileStorePos = ++m_curFileStorePos % fileStoreCount);
-                final FileStorage fileStore = m_fileStorageArray[nextFileStorePos];
+                final FileStorage fileStore = m_fileStorageArray[implGetNextFileStorePos()];
 
                 try {
                     // create new FileStore file
@@ -670,6 +648,13 @@ public class FileItemService implements IFileItemService {
 
     // - Implementation --------------------------------------------------------
 
+    /**
+     * @return
+     */
+    protected synchronized int implGetNextFileStorePos() {
+        return (m_curFileStorePos = ++m_curFileStorePos % m_fileStorageArray.length);
+    }
+
     /**
      * @param groupId
      * @param subGroupId
@@ -697,11 +682,13 @@ public class FileItemService implements IFileItemService {
 
     // - Members ---------------------------------------------------------------
 
+    final protected boolean m_valid;
+
     protected FileItemDatabase m_database = null;
 
     protected BidiMap m_fileStorageMap = null;
 
-    protected FileStorage[] m_fileStorageArray = null;
+    protected FileStorage[] m_fileStorageArray = new FileStorage[0];
 
     protected FileStoreRemover m_fileStoreRemover = null;
 
