@@ -1,52 +1,26 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
+
+
 package com.openexchange.fileitem.osgi;
 
 import java.io.File;
@@ -57,6 +31,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.LinkedHashSet;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
@@ -70,6 +45,7 @@ import org.osgi.util.tracker.ServiceTracker;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import com.google.common.base.Throwables;
 import com.openexchange.annotation.NonNull;
+import com.openexchange.annotation.Nullable;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.database.DatabaseService;
 import com.openexchange.database.Databases;
@@ -178,39 +154,17 @@ public class FileItemServiceActivator extends HousekeepingActivator {
         FileItemUtils.logInfo("Registering FileItemService");
 
         if (null != configService) {
+            // set spool dir
+            FileItemUtils.setSpoolDir(new File(configService.getProperty("com.openexchange.fileItem.spoolPath", "/tmp")));
+
+            // try to get configured filestore
             final Set<Integer> storeIdSet = implGetConfiguredStoreIds(configService);
-            final FileItemDatabase fileItemDatabase = implCreateFileItemDatabase(configService, storeIdSet);
+            final Optional<FileItemDatabase> fileItemDatabaseOpt = Optional.ofNullable(
+                implCreateFileItemDatabase(configService, storeIdSet));
+            final Optional<BidiMap> bidiMapOpt = Optional.ofNullable(
+                implGetFileStores(configService, storeIdSet));
 
-            if (null != fileItemDatabase) {
-                final long retryEndTimeMillis = System.currentTimeMillis() + 60000;
-                final long retryTimeoutMillis = 15000;
-                BidiMap fileStoreMap = null;
-
-                // try for a maximum of 60 seconds to retrieve the filestores;
-                // polling is necessary due to not deterministic initialization of
-                // the FileStorageInfoService with delayed setting of internal services
-                while (((null == (fileStoreMap = implGetFileStores(configService, storeIdSet))) || (fileStoreMap.size() < 1)) &&
-                        (System.currentTimeMillis() < retryEndTimeMillis)) {
-
-                    try {
-                        Thread.sleep(retryTimeoutMillis);
-                    } catch (@SuppressWarnings("unused") InterruptedException e) {
-                        // ok, try again until condition is met
-                    }
-                }
-
-                FileItemUtils.setSpoolDir(new File(configService.getProperty("com.openexchange.fileItem.spoolPath", "/tmp")));
-
-                if ((null != fileStoreMap) && (fileStoreMap.size() > 0)) {
-                    try {
-                        registerService(IFileItemService.class, m_fileItemService = new FileItemService(fileStoreMap, fileItemDatabase));
-                    } catch (Throwable e) {
-                        FileItemUtils.logExcp(e);
-                    }
-                } else if (FileItemUtils.isLogError()) {
-                    FileItemUtils.logError("No valid FileStore service interfaces found => FileItem service will not be available");
-                }
-            }
+            registerService(IFileItemService.class, m_fileItemService = new FileItemService(bidiMapOpt, fileItemDatabaseOpt));
         }
 
         if (null == m_fileItemService) {
@@ -224,7 +178,7 @@ public class FileItemServiceActivator extends HousekeepingActivator {
      * @param configService
      * @return
      */
-    private BidiMap implGetFileStores(@NonNull final ConfigurationService configService, @NonNull Set<Integer> storeIdSet) {
+    private @Nullable BidiMap implGetFileStores(@NonNull final ConfigurationService configService, @NonNull Set<Integer> storeIdSet) {
         final FileStorageService fileStoreService = getService(FileStorageService.class);
         BidiMap ret = new DualHashBidiMap();
 
@@ -270,7 +224,7 @@ public class FileItemServiceActivator extends HousekeepingActivator {
      * @param storeIdSet
      * @return
      */
-    private FileItemDatabase implCreateFileItemDatabase(@NonNull final ConfigurationService configService, @NonNull final Set<Integer> storeIdSet) {
+    private @Nullable FileItemDatabase implCreateFileItemDatabase(@NonNull final ConfigurationService configService, @NonNull final Set<Integer> storeIdSet) {
         FileItemDatabase ret = null;
 
         try {
@@ -307,10 +261,16 @@ public class FileItemServiceActivator extends HousekeepingActivator {
 
         // retrieve valid file store ids from config
         for (String curStoreIdStr : configService.getProperty("com.openexchange.fileItem.fileStoreIds", "-1").split(",")) {
-            final Integer curStoreId = Integer.valueOf(StringUtils.trim(curStoreIdStr));
+            if (StringUtils.isNotBlank(curStoreIdStr)) {
+                try {
+                    final Integer curStoreId = Integer.valueOf(StringUtils.trim(curStoreIdStr));
 
-            if (curStoreId.intValue() > 0) {
-                ret.add(curStoreId);
+                    if (curStoreId.intValue() > 0) {
+                        ret.add(curStoreId);
+                    }
+                } catch (@SuppressWarnings("unused") NumberFormatException e) {
+                    FileItemUtils.logError("Could not parse filestore id: " + curStoreIdStr);
+                }
             }
         }
 
@@ -319,7 +279,7 @@ public class FileItemServiceActivator extends HousekeepingActivator {
 
     // - Members ---------------------------------------------------------------
 
-    private ServiceTracker<DBMigrationMonitorService, DBMigrationMonitorService> m_dbMigrationMonitorTracker;
+    private ServiceTracker<DBMigrationMonitorService, DBMigrationMonitorService> m_dbMigrationMonitorTracker = null;
 
     private FileItemService m_fileItemService = null;
 
@@ -381,7 +341,6 @@ public class FileItemServiceActivator extends HousekeepingActivator {
                     }
                 }
 
-                con.commit();
                 rollback = 2;
             } catch (Exception e) {
                 throw FileStorageCodes.SQL_PROBLEM.create(e, e.getMessage());
@@ -390,8 +349,6 @@ public class FileItemServiceActivator extends HousekeepingActivator {
                     if (rollback == 1) {
                         Databases.rollback(con);
                     }
-
-                    Databases.autocommit(con);
                 }
 
                 releaseWritable(con, false);
