@@ -1,52 +1,26 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
+
+
 package com.openexchange.imageconverter.server.rest;
 
 import static org.apache.commons.lang.ArrayUtils.isNotEmpty;
@@ -101,6 +75,7 @@ import com.openexchange.imageconverter.impl.ImageConverter;
 import com.openexchange.imageconverter.impl.ImageConverterMBean;
 import com.openexchange.imageconverter.impl.ImageConverterUtils;
 import com.openexchange.imageconverter.impl.ImageProcessor;
+import com.openexchange.imageconverter.impl.Mutable;
 import com.openexchange.rest.services.JAXRSService;
 import com.openexchange.rest.services.annotation.Role;
 import com.openexchange.rest.services.annotation.RoleAllowed;
@@ -334,8 +309,17 @@ public class ImageConverterServerRest extends JAXRSService {
         if (m_running.get()) {
             try (final InputStream inputStm = implGetInputStream(multiPart)) {
                 if (null != inputStm) {
-                    try (final InputStream resultInputStm = m_imageConverter.cacheAndGetImage(imageKey, requestFormat, inputStm, context)) {
+                    final Mutable<Boolean> usedCache = new Mutable<>(Boolean.TRUE);
+
+                    try (final InputStream resultInputStm = (m_imageConverter instanceof ImageConverter) ?
+                            ((ImageConverter) m_imageConverter).cacheAndGetImage(imageKey, requestFormat, inputStm, usedCache, context) :
+                                m_imageConverter.cacheAndGetImage(imageKey, requestFormat, inputStm, context)) {
+
                         responseBuilder = implCreateFileResponse(resultInputStm);
+
+                        if (!usedCache.get()) {
+                            responseBuilder.header(HEADER_WARNING, MESSAGE_WARNING_NOCACHE);
+                        }
                     }
                 } else {
                     throw new IllegalArgumentException("IC #cacheAndGetImage server input file is not available");
@@ -373,10 +357,19 @@ public class ImageConverterServerRest extends JAXRSService {
         ResponseBuilder responseBuilder = null;
 
         if (m_running.get()) {
-            try (final InputStream sourceImageInputStream = implGetInputStream(multiPart)) {
-                if (null != sourceImageInputStream) {
-                    try (final MetadataImage metadaImage = m_imageConverter.cacheAndGetImageAndMetadata(imageKey, requestFormat, sourceImageInputStream, context)) {
+            try (final InputStream inputStm = implGetInputStream(multiPart)) {
+                if (null != inputStm) {
+                    final Mutable<Boolean> usedCache = new Mutable<>(Boolean.TRUE);
+
+                    try (final MetadataImage metadaImage = (m_imageConverter instanceof ImageConverter) ?
+                        ((ImageConverter) m_imageConverter).cacheAndGetImageAndMetadata(imageKey, requestFormat, inputStm, usedCache, context) :
+                            m_imageConverter.cacheAndGetImageAndMetadata(imageKey, requestFormat, inputStm, context)) {
+
                         responseBuilder = implCreateMetadataImageResponse(metadaImage);
+
+                        if (!usedCache.get()) {
+                            responseBuilder.header(HEADER_WARNING, MESSAGE_WARNING_NOCACHE);
+                        }
                     }
                 } else {
                     throw new IllegalArgumentException("IC WebService input file is not available");
@@ -527,7 +520,7 @@ public class ImageConverterServerRest extends JAXRSService {
      * @return
      */
     private static String implCreateImageConverterServerHtmlPage(final String[] content, @Nullable final IImageConverterMonitoring monitoring) {
-        final StringBuilder htmlBuilder = new StringBuilder(256);
+        final StringBuilder htmlBuilder = ImageConverterUtils.STR_BUILDER();
 
         htmlBuilder.append("<html>").
             append("<head><meta charset=\"UTF-8\"><title>").append(OX_IMAGECONVERTER_SERVER_TITLE).append("</title></head>").
@@ -557,7 +550,8 @@ public class ImageConverterServerRest extends JAXRSService {
                         final boolean isRatioMetric = metricsPropertyName.endsWith("Ratio");
 
                         try {
-                            final String numberString = new StringBuilder(256).append(method.invoke(monitoring)).toString();
+                            final String numberString = ImageConverterUtils.STR_BUILDER().
+                                append(method.invoke(monitoring)).toString();
 
                             if (NumberUtils.isNumber(numberString)) {
                                 htmlBuilder.append("<p>").append(metricsPropertyName).append(": ");
@@ -629,10 +623,10 @@ public class ImageConverterServerRest extends JAXRSService {
             jsonEngineStatus.put("imageProcessor", jsonImageProcessorStatus);
 
             // image formats
-            final ImageFormat[] usedImageFormats = imageConverter.getImageFormats();
+            final List<ImageFormat> usedImageFormats = imageConverter.getImageFormats();
 
             if (null != usedImageFormats) {
-                jsonEngineStatus.put("imageFormats", new JSONArray(Arrays.asList(usedImageFormats)));
+                jsonEngineStatus.put("imageFormats", new JSONArray(usedImageFormats));
             }
         } else {
             jsonEngineStatus.put("status", "n/a");
@@ -685,7 +679,7 @@ public class ImageConverterServerRest extends JAXRSService {
      * @return
      */
     private static ResponseBuilder implCreateFileResponse(final InputStream inputStm) {
-        return (null != inputStm) ? Response.ok(inputStm) : Response.status(Status.NOT_FOUND);
+        return (null != inputStm) ? Response.ok(inputStm) : Response.status(Status.NO_CONTENT);
     }
 
     /**
@@ -841,6 +835,14 @@ public class ImageConverterServerRest extends JAXRSService {
 
     final private static String[] OX_IMAGECONVERTER_SERVER_TERMINATED_RESPONSE_TEXT = { "Error Code: 99", " terminated!" };
 
+    final private static String HEADER_WARNING = "Warning";
+
+    final private static String MESSAGE_WARNING_NOCACHE = ImageConverterUtils.STR_BUILDER().
+        append(199).append(' ').
+        append("ImageConverter").append(' ').
+        append("\"Uncached\"").toString();
+
+
     // - Members ---------------------------------------------------------------
 
     final private  IImageConverter m_imageConverter;
