@@ -1,52 +1,26 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
+
+
 package com.openexchange.imageconverter.impl;
 
 import static com.openexchange.imageconverter.impl.ImageConverterUtils.LOG;
@@ -89,7 +63,7 @@ class ImageConverterJob implements Runnable, Closeable {
     /**
      * Initializes a new {@link ImageConverterJob}.
      * @param imageKey
-     * @param srcImageStm
+     * @param inputStm
      * @param imageFormats
      * @param queue
      * @param context
@@ -98,7 +72,7 @@ class ImageConverterJob implements Runnable, Closeable {
      */
     ImageConverterJob(
         @NonNull final String imageKey,
-        @NonNull final InputStream srcImageStm,
+        @NonNull final InputStream inputStm,
         @NonNull final List<ImageFormat> imageFormats,
         @NonNull ImageConverterQueue queue,
         @Nullable final String context) throws ImageConverterException {
@@ -108,13 +82,17 @@ class ImageConverterJob implements Runnable, Closeable {
         m_queue = queue;
         m_imageKey = imageKey;
         m_context = context;
+        m_inputFile = ImageConverterUtils.createTempFile();
+
+        if (null == m_inputFile) {
+            throw new ImageConverterException("IC ConversionJob error while creating input file: " + imageKey);
+        }
 
         try {
             // create tmp. input file from input stream, file is removed by calling close method
-            m_inputFile = File.createTempFile("oxic", ".tmp", ImageConverterConfig.IMAGECONVERTER_SPOOLPATH);
-            FileUtils.copyInputStreamToFile(srcImageStm, m_inputFile);
+            FileUtils.copyInputStreamToFile(inputStm, m_inputFile);
 
-            if ((null == m_inputFile) || (m_inputFile.length() < 1)) {
+            if (m_inputFile.length() < 1) {
                 throw new IOException("IC ConversionJob creation of job not possible due to missing or empty input file: " + imageKey);
             }
 
@@ -170,8 +148,8 @@ class ImageConverterJob implements Runnable, Closeable {
                 curTargetFormat = iter.next();
 
                 if (trace) {
-                    LOG.trace(ImageConverterUtils.STR_BUILDER().
-                        append("IC ConversionJob worker thread asynchronously processing ").
+                    LOG.trace(ImageConverterUtils.IC_STR_BUILDER().
+                        append("ConversionJob worker thread asynchronously processing ").
                         append(m_imageKey).append(": ").
                         append(curTargetFormat.getFormatString()).toString());
                 }
@@ -179,7 +157,7 @@ class ImageConverterJob implements Runnable, Closeable {
                 implProcessKey(curTargetFormat, true);
             }
         } catch (Exception e) {
-            LOG.error(ImageConverterUtils.STR_BUILDER().
+            LOG.error(ImageConverterUtils.IC_STR_BUILDER().
                 append(m_imageKey).append(": ").
                 append((null != curTargetFormat) ? curTargetFormat.getFormatString() :"null").toString(), e);
         } finally {
@@ -268,7 +246,6 @@ class ImageConverterJob implements Runnable, Closeable {
      */
     protected MetadataImage processRequestFormat(@NonNull final String requestFormat) throws ImageConverterException {
         ImageFormat requestTargetFormat = null;
-        MetadataImage ret = null;
 
         try {
             requestTargetFormat = ImageConverterUtils.getBestMatchingFormat(
@@ -280,26 +257,21 @@ class ImageConverterJob implements Runnable, Closeable {
                 throw new ImageConverterException("IC ConversionJob is not able to parse requested image format: " + requestFormat);
             }
 
-            boolean isAvailable = !m_formatsToProcess.containsKey(requestTargetFormat);
-
-            // no need to do anything, if requested format has already been processed
-            if (!isAvailable) {
-                isAvailable = implProcessKey(requestTargetFormat, false);
-            }
-
-            if (isAvailable) {
-                ret = new MetadataImage(ImageConverterUtils.readImageByTargetFormat(m_queue.getFileItemService(), m_imageKey, requestTargetFormat), m_imageMetadata);
+            // no need to do anything, if requested format has already been processed, otherwise process format
+            if (!m_formatsToProcess.containsKey(requestTargetFormat) || implProcessKey(requestTargetFormat, false)) {
+                return new MetadataImage(ImageConverterUtils.readImageByTargetFormat(
+                    m_queue.getFileItemService(), m_imageKey, requestTargetFormat),
+                    m_imageMetadata);
             }
         } catch (Exception e) {
-            throw new ImageConverterException(ImageConverterUtils.STR_BUILDER().
+            throw new ImageConverterException(ImageConverterUtils.IC_STR_BUILDER().
                 append(m_imageKey).
                 append(" / ").append(requestFormat).
-                append(" / ").append((null != requestTargetFormat) ?
-                    requestTargetFormat.getFormatString() :
-                        "null").toString(), e);
+                append(" / ").append((null != requestTargetFormat) ? requestTargetFormat.getFormatString() : "null").
+                toString(), e);
         }
 
-        return ret;
+        return null;
     }
 
     // - Implementation --------------------------------------------------------
@@ -372,7 +344,7 @@ class ImageConverterJob implements Runnable, Closeable {
                         final Dimension imageDimension = ret.getImageDimension();
                         final String formatName = ret.getImageFormatName();
 
-                        LOG.trace(ImageConverterUtils.STR_BUILDER().
+                        LOG.trace(ImageConverterUtils.IC_STR_BUILDER().
                             append("[").append(imageDimension.width).append('x').append(imageDimension.height).append("] ").
                             append(formatName).append(" image: ").
                             append(imageKey).toString());
@@ -431,13 +403,13 @@ class ImageConverterJob implements Runnable, Closeable {
             if (LOG.isTraceEnabled()) {
                 final Dimension imageDimension = imageMetadata.getImageDimension();
                 final String imageFormatShortName = imageMetadata.getImageFormatName();
-                final StringBuilder traceMsgBuilder = ImageConverterUtils.STR_BUILDER().
-                    append("IC image processor ").
+                final StringBuilder traceMsgBuilder = ImageConverterUtils.IC_STR_BUILDER().
+                    append("image processor ").
                     append(isAsync ? "a" : "").append("synchronously").
                     append(ret ? " scaled image to create " : " could not scale image to create ").
                     append(imageFormatShortName).append(" thumbnail for ");
 
-                LOG.trace(ImageConverterUtils.STR_BUILDER().
+                LOG.trace(ImageConverterUtils.IC_STR_BUILDER().
                     append(traceMsgBuilder).
                     append(implGetImageInformationString(imageKey, imageDimension, imageFormatShortName, targetFormatStr, usedProcessingFormat.getFormatString())).toString());
             }
@@ -445,8 +417,8 @@ class ImageConverterJob implements Runnable, Closeable {
             final Dimension imageDimension = imageMetadata.getImageDimension();
             final String imageFormatShortName = imageMetadata.getImageFormatName();
 
-            LOG.error(ImageConverterUtils.STR_BUILDER().
-                append("IC error while ").append(isAsync ? "a" : "").
+            LOG.error(ImageConverterUtils.IC_STR_BUILDER().
+                append("error while ").append(isAsync ? "a" : "").
                 append("synchronously creating ").append(imageFormatShortName).append(" thumbnail for ").
                 append(implGetImageInformationString(imageKey, imageDimension, imageFormatShortName, targetFormatStr, usedProcessingFormat.getFormatString())).toString());
         } finally {
@@ -484,8 +456,8 @@ class ImageConverterJob implements Runnable, Closeable {
                 final Dimension imageDimension = imageMetadata.getImageDimension();
                 final String imageFormatShortName = imageMetadata.getImageFormatName();
 
-                LOG.trace(ImageConverterUtils.STR_BUILDER().
-                    append("IC ").append(isAsync ? "a" : "").
+                LOG.trace(ImageConverterUtils.IC_STR_BUILDER().
+                    append(isAsync ? "a" : "").
                     append("synchronously created reference entry for ").
                     append(implGetImageInformationString(imageKey, imageDimension, imageFormatShortName, targetFormatStr, referenceFormatStr)).toString());
             }
@@ -495,8 +467,8 @@ class ImageConverterJob implements Runnable, Closeable {
             final Dimension imageDimension = imageMetadata.getImageDimension();
             final String imageFormatShortName = imageMetadata.getImageFormatName();
 
-            LOG.error(ImageConverterUtils.STR_BUILDER().
-                append("IC error while ").append(isAsync ? "a" : "").
+            LOG.error(ImageConverterUtils.IC_STR_BUILDER().
+                append("error while ").append(isAsync ? "a" : "").
                 append("synchronously creating reference entry for ").
                 append(implGetImageInformationString(imageKey, imageDimension, imageFormatShortName, targetFormatStr, referenceFormatStr)).toString());
         }
@@ -511,7 +483,7 @@ class ImageConverterJob implements Runnable, Closeable {
             m_queue.getFileItemService().remove(ImageConverterUtils.IMAGECONVERTER_GROUPID, imageKey, fileName);
         } catch (FileItemException e) {
             if (LOG.isTraceEnabled()) {
-                LOG.trace(ImageConverterUtils.STR_BUILDER().
+                LOG.trace(ImageConverterUtils.IC_STR_BUILDER().
                     append(imageKey).append(": ").
                     append(fileName).toString(), e);
             }
@@ -532,7 +504,6 @@ class ImageConverterJob implements Runnable, Closeable {
         for (final ImageFormat curFormat : imageFormats) {
             ImageFormat.ImageType targetFormatImageType = null;
 
-
             switch (curFormat.getFormatShortName()) {
                 case "jpg": {
                     targetFormatImageType = ImageFormat.ImageType.JPG;
@@ -547,7 +518,7 @@ class ImageConverterJob implements Runnable, Closeable {
                 case "auto":
                 default: {
                     targetFormatImageType = isAnimated ?
-                        // TODO (KA) add GIF format to possible target types
+                        // TODO (KA) add GIF format to possible target types???
                         ImageFormat.ImageType.PNG :
                             (isTransparent  ? ImageFormat.ImageType.PNG : ImageFormat.ImageType.JPG);
                     break;
