@@ -1,58 +1,31 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
+
+
 package com.openexchange.imageconverter.impl;
 
 import static com.openexchange.imageconverter.impl.ImageConverterUtils.LOG;
 import java.io.InputStream;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.LinkedList;
@@ -66,8 +39,6 @@ import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.ReentrantLock;
-import java.util.function.Predicate;
-import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang.ArrayUtils;
 import com.google.common.base.Throwables;
 import com.openexchange.annotation.NonNull;
@@ -82,7 +53,6 @@ import com.openexchange.imageconverter.api.IMetadataReader;
 import com.openexchange.imageconverter.api.ImageConverterException;
 import com.openexchange.imageconverter.api.ImageConverterPriority;
 import com.openexchange.imageconverter.api.ImageFormat;
-import com.openexchange.imageconverter.api.MetadataException;
 import com.openexchange.imageconverter.api.MetadataImage;
 import com.openexchange.imageconverter.impl.ImageProcessor.ImageProcessorStatus;
 
@@ -90,7 +60,7 @@ import com.openexchange.imageconverter.impl.ImageProcessor.ImageProcessorStatus;
  * {@link ImageConverterQueue}
  *
  * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
- * @since v7.10.0
+ * @since v8.0.0
  */
 class ImageConverterQueue extends Thread {
 
@@ -131,6 +101,7 @@ class ImageConverterQueue extends Thread {
                                     // close job, imageKey will be removed from processingMap as well
                                     implCloseJob(curJob, !isRunning());
 
+                                    // unlock imageKey mit UnlockMode.END_PROCESSING at end of processing
                                     ElementLocker.unlock(imageKey, ElementLock.UnlockMode.END_PROCESSING);
                                 }
                             }
@@ -172,48 +143,6 @@ class ImageConverterQueue extends Thread {
             return false;
         }
 
-        /**
-         * @param imageKey
-         * @return true if one key has been removed
-         */
-        public boolean removeJob(@NonNull final String imageKey) {
-            return implRemoveByPredicateOrAll((curImageKey) -> imageKey.equals(curImageKey));
-        }
-
-        /**
-         * @return true if at least one key has been removed
-         */
-        public boolean removeAllOpenJobs() {
-            return implRemoveByPredicateOrAll(null);
-        }
-
-        // - Implementation ----------------------------------------------------
-
-        /**
-         * @param predicateOrAll Remove all open jobs to execute, if given
-         *  Predicate is null and only the selected ones if Predicate is given
-         * @return true if at least one key has been removed
-         */
-        public boolean implRemoveByPredicateOrAll(@Nullable final Predicate<String> predicateOrAll) {
-            boolean ret = false;
-
-            synchronized (m_executorJobQueue) {
-                for (final Iterator<ImageConverterJob> iter = m_executorJobQueue.iterator(); iter.hasNext();) {
-                    final ImageConverterJob curJob = iter.next();
-
-                    if ((null != curJob) && ((null == predicateOrAll) || predicateOrAll.test(curJob.getImageKey()))) {
-                        ret = true;
-                        iter.remove();
-
-                        // close job, imageKey will be removed from processingMap as well
-                        implCloseJob(curJob, true);
-                    }
-                }
-            }
-
-            return ret;
-        }
-
         // - Members -----------------------------------------------------------
 
         final AtomicBoolean m_executorRunning = new AtomicBoolean(true);
@@ -229,7 +158,7 @@ class ImageConverterQueue extends Thread {
      * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
      * @since v7.10.2
      */
-    private class ImageConverterQueueEntry {
+    private static class ImageConverterQueueEntry {
 
         /**
          * Initializes a new {@link ImageConverterQueueEntry}.
@@ -286,7 +215,9 @@ class ImageConverterQueue extends Thread {
     /**
      * Initializes a new {@link ImageConverterQueue}.
      */
-    ImageConverterQueue(@NonNull final ImageConverter imageConverter,
+    ImageConverterQueue(
+        @NonNull final ImageConverter imageConverter,
+        @NonNull final ImageProcessor imageProcessor,
         @NonNull final IFileItemService fileItemService,
         @NonNull final IMetadataReader metadataReader) {
 
@@ -297,11 +228,8 @@ class ImageConverterQueue extends Thread {
         m_metadataReader = metadataReader;
 
         // create an ordered BidiMap to map ImageFormats to image format strings and vice versa
-        m_imageFormats = Arrays.asList(m_imageConverter.getImageFormats());
-        m_imageProcessor = new ImageProcessor(
-            ImageConverterConfig.IMAGECONVERTER_SEARCHPATH,
-            ImageConverterConfig.IMAGECONVERTER_USE_GRAPHICSMAGICK,
-            ImageConverterConfig.IMAGECONVERTER_IMAGE_USERCOMMENT);
+        m_imageFormats = m_imageConverter.getImageFormats();
+        m_imageProcessor = imageProcessor;
 
         for (final ImageConverterPriority curPriority : ImageConverterPriority.values()) {
             m_queuesImpl.put(curPriority, new ImageConverterQueueImpl(curPriority));
@@ -310,8 +238,8 @@ class ImageConverterQueue extends Thread {
         m_executor = new ImageConverterJobExecutor(ImageConverterConfig.IMAGECONVERTER_THREAD_COUNT);
 
         if (LOG.isInfoEnabled()) {
-            LOG.info(ImageConverterUtils.STR_BUILDER().
-                append("IC conversion queue started (thread count: ").
+            LOG.info(ImageConverterUtils.IC_STR_BUILDER().
+                append("conversion queue started (thread count: ").
                 append(ImageConverterConfig.IMAGECONVERTER_THREAD_COUNT).
                 append(", queue entries: ").append(ImageConverterConfig.IMAGECONVERTER_QUEUE_LENGTH).
                 append(')').toString());
@@ -362,9 +290,9 @@ class ImageConverterQueue extends Thread {
                         implCloseJob(job, true);
                     }
                 }
-            } catch (@SuppressWarnings("unused") Exception e) {
+            } catch (Exception e) {
                 if (trace) {
-                    LOG.trace("IC queue received exception while taking next job from queue and forwarding to executor: {}", Throwables.getRootCause(e));
+                    LOG.trace("IC queue received exception while taking next job from queue and forwarding to executor: {}", Throwables.getRootCause(e).getMessage());
                 }
             }
         }
@@ -413,49 +341,115 @@ class ImageConverterQueue extends Thread {
 
     /**
      * @param imageKey
-     * @param srcImageStm
-     * @param context
+     * @return true in case a job is currently scheduled for processing or processed
      */
-    void cache(@NonNull final String imageKey,
-        @NonNull final InputStream srcImageStm,
-        @Nullable final String context,
-        @Nullable ImageConverterPriority priority) throws ImageConverterException {
-
-        if (!m_imageConverter.isAvailable(imageKey)) {
-            implCache(imageKey, srcImageStm, null, context, null);
-        }
+    public boolean isProcessing(@NonNull final String imageKey) {
+        return m_processingMap.containsKey(imageKey);
     }
 
     /**
-     * @param imageKey
-     * @param srcImageStm
-     * @param context
-     * @return
+     * @param imageKey The image key.
+     * @param srcImageStm The image input data.
+     * @param priority The priority of job to be queued.
+     * @param requestFormat The optional request format for which to return an image synchronously.
+     *  This paramter might be null in which case only a cache job is scheduled and null is returned.
+     * @param context The optional context data.
+     * @param removeEntryInCaseOfError Removes just created job results in case of an error.
+     * @return The requested MetadataImage or null if requestFormat is null.
+     * @throws ImageConverterException
      */
-    MetadataImage cacheAndGetMetadataImage(@NonNull final String imageKey,
+    MetadataImage cacheAndGetMetadataImage(
+        @NonNull final String imageKey,
         @NonNull final InputStream srcImageStm,
-        @NonNull final String requestFormat,
+        @NonNull final ImageConverterPriority priority,
+        @Nullable final String requestFormat,
         @Nullable final String context,
-        @Nullable final ImageConverterPriority priority) throws ImageConverterException {
+        boolean removeEntryInCaseOfError) throws ImageConverterException {
 
+        final boolean hasRequestFormat = (null != requestFormat);
         MetadataImage ret = null;
 
+        // don't create a new job with same imageKey if such a job has been already created or is currently processed
+        ElementLocker.lock(imageKey, ElementLock.LockMode.WAIT_IF_PROCESSED);
+
         try {
-            ret = getMetadataImage(imageKey, requestFormat);
-        } catch (ImageConverterException e) {
-            LOG.trace("IC not yet able to read MetadataImage, waiting for processed key first", e);
+            // When a previous job is already created or finished, but waiting in a queue to be processed
+            // it is not locked during queue time => we don't want to create a new job in these cases, so
+            // that we check here if already finished
+            if (!m_imageConverter.isAvailable(imageKey)) {
+                // begin processing out of the lock state of the previous lock
+                ElementLocker.lock(imageKey, ElementLock.LockMode.BEGIN_PROCESSING);
+
+                ImageConverterJob job = null;
+
+                try {
+                    job = new ImageConverterJob(imageKey, srcImageStm, m_imageFormats, this, context);
+
+                    if (hasRequestFormat && (null == (ret = job.processRequestFormat(requestFormat)))) {
+                        // resource warning has been checked => job is closed in catch block
+                        throw new ImageConverterException("IC error while creating/processing new ImageConverterJob");
+                    }
+
+                    int updatedQueueSize = 0;
+
+                    synchronized (m_queuesImpl) {
+                        final LinkedList<ImageConverterJob> queue = m_queuesImpl.get(priority);
+
+                        // add job to appropriate, prioritized queue
+                        // reset measurement time of job in order to retrieve the
+                        // queue time, after which the job is taken from queue
+                        job.initMeasurementTime();
+
+                        queue.add(job);
+                        updatedQueueSize = queue.size();
+                    }
+
+                    ImageConverterUtils.IC_MONITOR.setQueueCount(priority, updatedQueueSize);
+
+                    if (LOG.isTraceEnabled()) {
+                        LOG.trace(ImageConverterUtils.IC_STR_BUILDER().
+                            append("entry count after put to ").
+                            append(priority.name().toUpperCase()).
+                            append(" queue: ").append(updatedQueueSize).toString());
+                    }
+
+                    m_processingMap.put(imageKey, job);
+                    implSignalAvailability();
+
+                    // job has just been added to queue => unlock job now, it will be locked
+                    // again when taken from queue for processing; final unlocking of imageKey
+                    // with UnlockMode.END_PROCESSING will happen when job is closed
+                    ElementLocker.unlock(imageKey);
+                } catch (ImageConverterException e) {
+                    LOG.warn("IC removing entry after occurence of an error: {}", imageKey);
+
+                    if (null != job) {
+                        implCloseJob(job, removeEntryInCaseOfError);
+                    }
+
+                    // unlock the imageKey with UnlockMode.END_PROCESSING in case of an error
+                    ElementLocker.unlock(imageKey, ElementLock.UnlockMode.END_PROCESSING);
+
+                    throw e;
+                }
+            }
+
+            if (hasRequestFormat) {
+                try {
+                    // if requestFormat is given, return any result we already have so far
+                    return (null != ret) ? ret : implGetMetadataImage(imageKey, requestFormat);
+                } catch (ImageConverterException e) {
+                    LOG.warn(ImageConverterUtils.IC_STR_BUILDER().
+                        append("queue could not get MetadataImage for ").
+                        append(imageKey).append(": ").append(requestFormat).toString(),
+                        Throwables.getRootCause(e));
+                }
+            }
+        } finally {
+            ElementLocker.unlock(imageKey);
         }
 
-        if (null == ret) {
-            return implCache(
-                imageKey,
-                srcImageStm,
-                requestFormat,
-                context,
-                (null != priority) ? priority : ImageConverterPriority.MEDIUM);
-        }
-
-        return ret;
+        return null;
     }
 
     /**
@@ -465,82 +459,87 @@ class ImageConverterQueue extends Thread {
      * @return
      */
     MetadataImage getMetadataImage(@NonNull final String imageKey, @NonNull final String requestFormat) throws ImageConverterException {
-        if (m_imageConverter.isAvailable(imageKey)) {
-            ImageFormat requestTargetFormat = null;
+        ImageFormat requestTargetFormat = null;
+
+        try {
             final boolean trace = LOG.isTraceEnabled();
 
-            try {
-                final IMetadata imageMetadata = getMetadata(imageKey);
-
-                if (null == imageMetadata) {
-                    throw new MetadataException("IC could not read metadata");
-                }
-
-                // lock element in processing mode (just lock, no condition wait)
-                // first and check, if appropriate job is still queued or processed
+            // try to lock key and get result from already queued job
+            if (ElementLocker.lock(imageKey, ElementLock.LockMode.TRY_LOCK)) {
                 try {
-                    ElementLocker.lock(imageKey, ElementLock.LockMode.STANDARD);
+                    ImageConverterQueueEntry queueEntry = null;
 
-                    final ImageConverterQueueEntry queueEntry = implGetQueuedJob(imageKey, false);
+                    // try to get job from queue
+                    synchronized (m_queuesImpl) {
+                        ImageConverterJob curJob = null;
 
-                    // job is queued =>
-                    // promote job to highest priority queue, process requested format
-                    // and return result immediately;
-                    // job is asynchronously processed afterwards
+                        // iterate over all prioritzed queues to check if there's a waiting job
+                        for (final ImageConverterPriority curPriority : PRIORITY_ORDER_HIGH_TO_LOW) {
+                            final LinkedList<ImageConverterJob> implQueue = m_queuesImpl.get(curPriority);
+                            final Iterator<ImageConverterJob> iter = implQueue.iterator();
+
+                            while (iter.hasNext()) {
+                                curJob = iter.next();
+
+                                if ((null != curJob) && imageKey.equals(curJob.getImageKey())) {
+                                    // move job to highest priority queue
+                                    if (PRIORITY_HIGHEST != curPriority) {
+                                        iter.remove();
+                                        m_queuesImpl.get(PRIORITY_HIGHEST).add(curJob);
+                                    }
+
+                                    queueEntry = new ImageConverterQueueEntry(curJob, curPriority);
+                                    break;
+                                }
+                            }
+
+                            if (null != queueEntry) {
+                                break;
+                            }
+                        }
+                    }
+
+                    // job is queued => return requested format immediately;
+                    // the job is asynchronously processed afterwards
                     if (null != queueEntry) {
                         if (trace) {
-                            LOG.trace("IC queue synchronously processes job in user request: {}", imageKey);
+                            LOG.trace("IC queue synchronously processes job in getMetadataImage user request: {}", imageKey);
                         }
 
-                        // change priority for already scheduled job
-                        implChangeJobPriority(imageKey, ImageConverterPriority.highest());
-
                         // get MetadataImage from job directly
-                        return queueEntry.getJob().processRequestFormat(requestFormat);
+                        final MetadataImage ret = queueEntry.getJob().processRequestFormat(requestFormat);
+
+                        implSignalAvailability();
+
+                        return ret;
                     }
                 } finally {
-                    ElementLocker.unlock(imageKey, ElementLock.UnlockMode.STANDARD);
-                }
-
-                try {
-                    // image key is available in general, but not queued anymore =>
-                    // wait until outstanding job has been processed completely, if necessary
-                    if (trace) {
-                        LOG.trace("IC queue gets image/waits for finish of asynchronous job processing in user request: {}", imageKey);
-                    }
-
-                    ElementLocker.lock(imageKey, ElementLock.LockMode.WAIT_IF_PROCESSED);
-
-                    // get all processed image formats and determine target request format
-                    final List<ImageFormat> availableImageFormats = implGetAvailableImageKeyFormats(imageKey);
-
-                    // if there're no available formats, the entry must be invalid
-                    if (0 != availableImageFormats.size()) {
-                        // determine target format to retrieve
-                        requestTargetFormat = ImageConverterUtils.getBestMatchingFormat(
-                            availableImageFormats,
-                            imageMetadata,
-                            ImageFormat.parseImageFormat(requestFormat));
-
-                        if (null != requestTargetFormat) {
-                            return new MetadataImage(ImageConverterUtils.readImageByTargetFormat(m_fileItemService, imageKey, requestTargetFormat), imageMetadata);
-                        }
-                    }
-
-                    throw new ImageConverterException("IC could not determine any valid MetadataImage based on given request format");
-                } finally {
-                    ElementLocker.unlock(imageKey, ElementLock.UnlockMode.STANDARD);
-                }
-            } catch (Exception e) {
-                if (trace) {
-                    LOG.trace(ImageConverterUtils.STR_BUILDER().
-                        append("IC queue could not get MetadataImage for ").
-                        append(imageKey).append(": ").
-                        append(requestFormat).append(" / ").
-                        append((null != requestTargetFormat) ? requestTargetFormat.getFormatString() : "n/a").toString(),
-                        Throwables.getRootCause(e));
+                    ElementLocker.unlock(imageKey);
                 }
             }
+
+            // image key is available in general, but not queued anymore =>
+            // wait until outstanding job has been processed completely, if necessary
+            if (trace) {
+                LOG.trace("IC queue gets image/waits for finish of asynchronous job processing in getMetadataImage user request: {}", imageKey);
+            }
+
+            ElementLocker.lock(imageKey, ElementLock.LockMode.WAIT_IF_PROCESSED);
+
+            try {
+                if (m_imageConverter.isAvailable(imageKey)) {
+                    return implGetMetadataImage(imageKey, requestFormat);
+                }
+            } finally {
+                ElementLocker.unlock(imageKey);
+            }
+        } catch (Exception e) {
+            LOG.warn(ImageConverterUtils.IC_STR_BUILDER().
+                append("queue could not get MetadataImage for ").
+                append(imageKey).append(": ").
+                append(requestFormat).append(" / ").
+                append((null != requestTargetFormat) ? requestTargetFormat.getFormatString() : "n/a => Removing key!").toString(),
+                Throwables.getRootCause(e));
         }
 
         return null;
@@ -548,78 +547,32 @@ class ImageConverterQueue extends Thread {
 
     /**
      * @param imageKey
-     * @param srcImageStm
+    * @param srcImageStm
      * @param context
      * @return
      */
     IMetadata getMetadata(@NonNull final String imageKey) throws ImageConverterException {
-        if (m_imageConverter.isAvailable(imageKey)) {
-            // job is queued =>
-            // get metadadata from still running job
-            final ImageConverterJob job = m_processingMap.get(imageKey);
+        // if job is currently processed => get metadadata from still running job
+        final ImageConverterJob job = m_processingMap.get(imageKey);
 
-            // Resource warning has been checked:
-            // job is closed when runnable finishes;
-            // try to get getmetada from still scheduled or processing job
-            if (null != job) {
-                return job.getImageMetadata();
-            }
+        if (null != job) {
+            return job.getImageMetadata();
+        }
 
-            // image key is available in general, but not in queue anymore =>
-            // read from persistent data
-            ElementLocker.lock(imageKey, ElementLock.LockMode.WAIT_IF_PROCESSED);
+        // try to read metada from existing entry
+        ElementLocker.lock(imageKey, ElementLock.LockMode.WAIT_IF_PROCESSED);
 
-            try {
+        try {
+            if (m_imageConverter.isAvailable(imageKey)) {
                 return ImageConverterUtils.readMetadata(m_fileItemService, m_metadataReader, imageKey);
-            } finally {
-                ElementLocker.unlock(imageKey, ElementLock.UnlockMode.STANDARD);
             }
+        } finally {
+            ElementLocker.unlock(imageKey);
         }
 
         return null;
     }
 
-    /**
-     * @param imageKey
-     * @throws ImageConverterException
-     */
-    void removeImageConverterJob(@NonNull final String imageKey) {
-        ElementLocker.lock(imageKey, ElementLock.LockMode.STANDARD);
-
-        // lock element in processing mode (just lock, no condition wait)
-        // first and check, if appropriate job is still queued or processed
-        try {
-            final boolean trace = LOG.isTraceEnabled();
-            String removeLocation = null;
-
-            if (null != implGetQueuedJob(imageKey, true)) {
-                if (trace) {
-                    removeLocation = "priority queue";
-                }
-            } else {
-                // remove job from executor queue
-                // removal of persistent data should be handled by
-                // caller providing a list of imageKeys to remove
-                // for performance reasons
-                m_executor.removeJob(imageKey);
-
-                if (trace) {
-                    removeLocation = "executor queue";
-                }
-            }
-
-            if (trace) {
-                LOG.trace(ImageConverterUtils.STR_BUILDER().
-                    append("IC queue removed pending job from ").
-                    append(removeLocation).append(": ").
-                    append(imageKey).toString());
-            }
-        } finally {
-            ElementLocker.unlock(imageKey, ElementLock.UnlockMode.END_PROCESSING);
-        }
-
-   }
-
     /**
      *
      */
@@ -667,78 +620,6 @@ class ImageConverterQueue extends Thread {
 
     // - Implementation --------------------------------------------------------
 
-    /**
-     * @param imageKey
-     */
-    protected MetadataImage implCache(@NonNull final String imageKey,
-        @NonNull final InputStream srcImageStm,
-        @Nullable final String requestFormat,
-        @Nullable final String context,
-        @Nullable final ImageConverterPriority priority) throws ImageConverterException {
-
-        // we need to ensure that no other job with the same
-        // imageKey is currently running => wait until processed
-        ElementLocker.lock(imageKey, ElementLock.LockMode.WAIT_IF_PROCESSED);
-
-        try {
-            // begin processing out of the lock state of the previous lock
-            ElementLocker.lock(imageKey, ElementLock.LockMode.BEGIN_PROCESSING);
-        } finally {
-            // unlock the first waiting lock now, the BEGIN_PROCESSING lock
-            // is still active and will be unlocked temporarily after
-            // queuing and initial request processing;
-            // final END_PROCESSING unlock will happen in an async. way
-            ElementLocker.unlock(imageKey, ElementLock.UnlockMode.STANDARD);
-        }
-
-        if (m_processingMap.containsKey(imageKey)) {
-            LOG.warn("IC queue processing map already contains imageKey while start queueing: {}", imageKey);
-        }
-
-        ImageConverterJob job = null;
-        MetadataImage ret = null;
-        boolean errorOccured = false;
-
-        try {
-            job = new ImageConverterJob(imageKey, srcImageStm, m_imageFormats, this, context);
-
-            if (null != requestFormat) {
-                errorOccured = (null == (ret = job.processRequestFormat(requestFormat)));
-            }
-
-            // add job to appropriate queue for asynchronous
-            // processing, if everything went well so far
-            if (!errorOccured) {
-                implAdd(job, (null != priority) ? priority : ImageConverterPriority.BACKGROUND);
-            }
-        } catch (ImageConverterException e) {
-            // job is closed in finally block
-            errorOccured = true;
-            throw e;
-        } finally {
-            if (errorOccured) {
-                // close job in case of error =>
-                // the imageKey will be removed from processingMap as well
-                if (null != job) {
-                    implCloseJob(job, true);
-                }
-
-                LOG.warn("IC removed entry after occurence of an error: {}", imageKey);
-            }
-
-            // unlock image key with unlock mode based on error status
-            ElementLocker.unlock(imageKey,
-                errorOccured ?
-                    ElementLock.UnlockMode.END_PROCESSING :
-                        ElementLock.UnlockMode.STANDARD);
-        }
-
-        // Resource warning has been checked:
-        // job is already closed or will be closed
-        // when ImageConverter Job finishes
-        return ret;
-    }
-
     /**
      * @param imageKey
      * @return
@@ -785,7 +666,7 @@ class ImageConverterQueue extends Thread {
             LOG.trace("IC error while removing imageKey when closing Runnable: {}", imageKey, e);
         } finally {
             // close job itself
-            IOUtils.closeQuietly(job);
+            ImageConverterUtils.close(job);
 
             // remove imageKey from processing map
             if (null != imageKey) {
@@ -798,88 +679,35 @@ class ImageConverterQueue extends Thread {
 
     // - Implementation --------------------------------------------------------
 
-    /**
-     * @param priority
-     * @param job
-     */
-    void implAdd(@NonNull ImageConverterJob job, @NonNull final ImageConverterPriority priority) {
-        int updatedQueueSize = 0;
+    private MetadataImage implGetMetadataImage(@NonNull final String imageKey, @NonNull final String requestFormat) throws ImageConverterException {
+        // get all processed image formats and determine target request format
+        final List<ImageFormat> availableImageFormats = implGetAvailableImageKeyFormats(imageKey);
 
-        // add job to appropriate, prioritized
-        // reset measurement time of job in order to retrieve the
-        // queue time, after which the job is taken from queue
-        job.initMeasurementTime();
+        // if there're no available formats, the entry must be invalid
+        if (availableImageFormats.size() > 0) {
+            final IMetadata imageMetadata = ImageConverterUtils.readMetadata(m_fileItemService, m_metadataReader, imageKey);
 
-        synchronized (m_queuesImpl) {
-            final ImageConverterQueueImpl queue = m_queuesImpl.get(priority);
+            if (null == imageMetadata) {
+                return null;
+            }
 
-            queue.add(job);
-            updatedQueueSize = queue.size();
-        }
+            // determine target format to retrieve
+            final ImageFormat requestTargetFormat = ImageConverterUtils.getBestMatchingFormat(
+                availableImageFormats,
+                imageMetadata,
+                ImageFormat.parseImageFormat(requestFormat));
 
-        ImageConverterUtils.IC_MONITOR.setQueueCount(priority, updatedQueueSize);
-
-        if (LOG.isTraceEnabled()) {
-            LOG.trace(ImageConverterUtils.STR_BUILDER().
-                append("IC entry count after put to ").
-                append(priority.name().toUpperCase()).
-                append(" queue: ").append(updatedQueueSize).toString());
-        }
-
-        m_processingMap.put(job.getImageKey(), job);
-        implSignalAvailability();
-    }
-
-    /**
-     * @param job
-     * @param newPriority
-     * @return
-     */
-    private boolean implChangeJobPriority(@NonNull final String imageKey, @NonNull final ImageConverterPriority newPriority) {
-        boolean priorityChange = false;
-
-        synchronized (m_queuesImpl) {
-            // iterate over all prioritized queues from highest to lowest
-            // priority, in order to retrieve the next job candidate
-            for (final ImageConverterPriority curPriority : PRIORITY_LOW_TO_HIGH) {
-                // don't remove/add from/to queue with same priority
-                if (curPriority != newPriority) {
-                    final ImageConverterQueueImpl curQueue = m_queuesImpl.get(curPriority);
-                    final Iterator<ImageConverterJob> iter = curQueue.iterator();
-
-                    while (iter.hasNext()) {
-                        // Resource warning has been checked:
-                        // job is closed when runnble finishes
-                        final ImageConverterJob curJob = iter.next();
-
-                        if (imageKey.equals(curJob.getImageKey())) {
-                            // move from old priority queue into new priority queue
-                            iter.remove();
-                            m_queuesImpl.get(newPriority).add(curJob);
-
-                            if (LOG.isTraceEnabled()) {
-                                LOG.trace(ImageConverterUtils.STR_BUILDER().
-                                        append("IC entry has been moved from ").
-                                        append(curPriority.name().toUpperCase()).
-                                        append(" queue to ").
-                                        append(newPriority.name().toUpperCase()).
-                                        append(" queue: ").
-                                        append(imageKey).toString());
-                            }
-
-                            priorityChange = true;
-                            break;
-                        }
-                    }
+            if (null != requestTargetFormat) {
+                try {
+                    return new MetadataImage(ImageConverterUtils.readImageByTargetFormat(
+                        m_fileItemService, imageKey, requestTargetFormat), imageMetadata);
+                } catch (Exception e) {
+                    throw new ImageConverterException("IC could not create MetadataImage: " + imageKey, e);
                 }
             }
         }
 
-        if (priorityChange) {
-            implSignalAvailability();
-        }
-
-        return priorityChange;
+        throw new ImageConverterException("IC could not determine any valid MetadataImage: " + imageKey);
     }
 
     /**
@@ -898,42 +726,6 @@ class ImageConverterQueue extends Thread {
         return false;
     }
 
-    /**
-     * @param imageKey
-     * @return
-     */
-    private ImageConverterQueueEntry implGetQueuedJob(@NonNull final String imageKey, boolean endProcessing) {
-        synchronized (m_queuesImpl) {
-            ImageConverterJob curJob = null;
-
-            // iterate over all prioritzed queues to check if there's a waiting job
-            for (final ImageConverterPriority curPriority : PRIORITY_ORDER_HIGH_TO_LOW) {
-                final ImageConverterQueueImpl implQueue = m_queuesImpl.get(curPriority);
-
-                for (final Iterator<ImageConverterJob> iter = implQueue.iterator(); iter.hasNext();) {
-                    // resource warning has been checked:
-                    // job is closed later or it will be closed
-                    // after it will have been executed as FutureTask
-                    if ((null != (curJob = iter.next())) && imageKey.equals(curJob.getImageKey())) {
-                        if (endProcessing) {
-                            iter.remove();
-
-                            // close job, imageKey will be removed from processingMap as well
-                            implCloseJob(curJob, false);
-                        }
-
-                        // resource warning has been checked, job was either
-                        // closed already or it will be closed after it will
-                        // have been executed as FutureTask
-                        return new ImageConverterQueueEntry(curJob, curPriority);
-                    }
-                }
-            }
-        }
-
-        return null;
-    }
-
     /**
      * @return
      */
@@ -948,8 +740,8 @@ class ImageConverterQueue extends Thread {
 
                 if (!curQueue.isEmpty() && (null != (job = curQueue.remove()))) {
                     if (LOG.isTraceEnabled()) {
-                        LOG.trace(ImageConverterUtils.STR_BUILDER().
-                                append("IC entry count after take from ").
+                        LOG.trace(ImageConverterUtils.IC_STR_BUILDER().
+                                append("entry count after take from ").
                                 append(curPriority.name().toUpperCase()).
                                 append(" queue: ").append(curQueue.size()).toString());
                     }
@@ -999,6 +791,11 @@ class ImageConverterQueue extends Thread {
      */
     final private static ImageConverterPriority[] PRIORITY_LOW_TO_HIGH = (ImageConverterPriority[]) ArrayUtils.clone(PRIORITY_ORDER_HIGH_TO_LOW);
 
+    /**
+     * PRIORITY_HIGHEST
+     */
+    final private static ImageConverterPriority PRIORITY_HIGHEST = ImageConverterPriority.highest();
+
     static {
         ArrayUtils.reverse(PRIORITY_LOW_TO_HIGH);
     }
