@@ -1,61 +1,38 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
+
+
 package com.openexchange.imageconverter.impl;
 
 import java.awt.Dimension;
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
+import java.io.Closeable;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.nio.file.Files;
+import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
@@ -64,12 +41,15 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 import java.util.TreeSet;
+import java.util.concurrent.atomic.AtomicLong;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang.StringUtils;
 import org.json.JSONObject;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import com.google.common.base.Throwables;
 import com.openexchange.annotation.NonNull;
+import com.openexchange.annotation.Nullable;
 import com.openexchange.imageconverter.api.FileItemException;
 import com.openexchange.imageconverter.api.IFileItem;
 import com.openexchange.imageconverter.api.IFileItemReadAccess;
@@ -168,10 +148,49 @@ public class ImageConverterUtils {
     /**
      * @return
      */
-    public  static StringBuilder STR_BUILDER() {
+    public static StringBuilder STR_BUILDER() {
         return new StringBuilder(256);
     }
 
+    /**
+     * @return
+     */
+    public static StringBuilder IC_STR_BUILDER() {
+        return STR_BUILDER().append("IC ");
+    }
+
+    /**
+    * @return
+    */
+    public static @Nullable File createTempFile() {
+        try {
+            return File.createTempFile(TEMP_COUNTER.incrementAndGet() + "oxic", ".tmp", ImageConverterConfig.IMAGECONVERTER_SPOOLPATH);
+        } catch (IOException e) {
+            LOG.error("IC can not create temp. file: {}", Throwables.getRootCause(e).getMessage());
+        }
+        return null;
+    }
+
+    /**
+     * @return
+     */
+    public static @Nullable File createTempDir() {
+        try {
+            final Path tmpDirPath = Files.createTempDirectory(
+                ImageConverterConfig.IMAGECONVERTER_SPOOLPATH.toPath(),
+                TEMP_COUNTER.incrementAndGet() + "oxicd");
+
+            if (null != tmpDirPath) {
+                return tmpDirPath.toFile();
+            }
+        } catch (Exception e) {
+            LOG.error("IC can not create temp. directory: {}", Throwables.getRootCause(e).getMessage());
+        }
+
+        return null;
+
+    }
+
     /**
      * @param availableFormats
      * @param requestFormat
@@ -249,12 +268,14 @@ public class ImageConverterUtils {
                 ret = imageFormats.get(imageFormats.size() - 1);
 
                 // this should not happen at all, if a valid ImageFormat list is provided
-                LOG.error(STR_BUILDER().append("IC found no best matching image format for requested format, using target format: ").
-                    append(requestFormat.getFormatString()).append(" => ").append(ret.getFormatString()).toString());
+                LOG.error(IC_STR_BUILDER().
+                    append("found no best matching image format for requested format, using target format: ").
+                    append(requestFormat.getFormatString()).append(" => ").
+                    append(ret.getFormatString()).toString());
             }
 
             if (trace) {
-                LOG.trace(STR_BUILDER().append("IC found best matching image format for requested format: ").
+                LOG.trace(IC_STR_BUILDER().append("found best matching image format for requested format: ").
                     append(requestFormat.getFormatString()).append(" => ").append(ret.getFormatString()).toString());
             }
         } else if (LOG.isWarnEnabled()) {
@@ -264,23 +285,6 @@ public class ImageConverterUtils {
         return ret;
     }
 
-    /**
-     * @param inputFile
-     * @return
-     */
-    public static String getIM4JInput(File inputFile) {
-        return ((null == inputFile) ? "logo:" : inputFile.getAbsolutePath());
-    }
-
-    /**
-     * @param outputFile
-     * @param imageFormat
-     * @return
-     */
-    public static String getIM4JOutput(@NonNull File outputFile, @NonNull final ImageFormat imageFormat) {
-        return STR_BUILDER().append(imageFormat.getFormatShortName()).append(':').append(outputFile.getAbsolutePath()).toString();
-    }
-
     /**
      * @param extent
      * @param positiveOnly
@@ -452,6 +456,19 @@ public class ImageConverterUtils {
         }
     }
 
+    /**
+     * @param closeable
+     */
+    public static void close(final Closeable closeable) {
+        if (null != closeable) {
+            try {
+                closeable.close();
+            } catch (IOException e) {
+                LOG.error(Throwables.getRootCause(e).getMessage());
+            }
+        }
+    }
+
     // - Implementation --------------------------------------------------------
 
     /**
@@ -503,6 +520,8 @@ public class ImageConverterUtils {
     final private static int TEST_SHRINKONLY = 2;
     final private static int TEST_FORMATSHORTNAME = 3;
 
+    final private static AtomicLong TEMP_COUNTER = new AtomicLong(0);
+
     static {
         // sort list of to be registered IMAGECONVERTER_KEYS custom keys
         Arrays.sort(IMAGECONVERTER_KEYS);
